@@ -25,15 +25,19 @@
         *** Various combinations of the above
 """
 
-from .basic import isNumber,isInteger,isComplex,Number
+from .basic import isNumber,isInteger,isComplex,Number,product
 from . import polynomials as pl
 from .trigonometric import arccos
 from .powers import sqrt
 from .num_theory import complex_polar
 import re
 from typing import Union,Any,Dict,Tuple
+from . import random as rn
 
 WHITESPACE = ' '
+EXCEPTION_MSG : callable = lambda method : f"{method} method was not defined for at least one element in the Matrix.\n(Caused from performing the {method} operation on two elements whose {method} method returns NotImplemented"
+EXCEPTION_MSG_v : callable = lambda method : f"{method} method was not defined for at least one element in the Vector.\n(Caused from performing the {method} operation on two elements whose {method} method returns NotImplemented"
+
 
 
 epsilon = pl.x #epsilon = lamda 
@@ -42,9 +46,6 @@ POLY = type(epsilon) #<class 'pythematics.polynomials.Polynomial'>
 
 class Vector:
     def __init__(self,array):
-        for item in array:
-            if not isNumber(item) and type(item) != complex and type(item) != POLY:
-                raise ValueError("Vector arguments must be a list of {} , {} , {} or {} not {}".format(int,float,complex,POLY,type(item)))
         self.matrix = array
         self.rows = len(self.matrix)
         self.collumns = 1
@@ -56,6 +57,9 @@ class Vector:
             if 'deg' in str(item): #Check if polynomial
                 item = re.sub(r"\s+",'',str(item).split(":")[-1])
                 item = f'({item})'
+            elif 'Multivariable' in str(item): #Check if multinomial
+                item = re.sub(r"\s+",'',str(item).split(":")[-1])
+                item = f'{item}'
             print(f'R{i}| {item:>3}')
             i+=1
         s2 = "\n{} x {} Vector array\n".format(self.rows,self.collumns)
@@ -78,7 +82,13 @@ class Vector:
             for i in range(self.getSize()):
                 empty.append(value.getMatrix()[i] + self.getMatrix()[i])
             return Vector(empty)
-        return NotImplemented
+        try:
+            return self.forEach(lambda y : y + value)
+        except:
+            raise EXCEPTION_MSG_v('__add__')
+
+    def __radd__(self,value):
+        return self.__add__(value)
 
     def __sub__(self,value):
         empty = []
@@ -88,8 +98,13 @@ class Vector:
             for i in range(self.getSize()):
                 empty.append(value.getMatrix()[i] - self.getMatrix()[i])
             return Vector(empty)
-        else:
-            return NotImplemented
+        try:
+            return self.forEach(lambda y : y - value)
+        except:
+            raise ValueError(EXCEPTION_MSG_v("__sub__"))
+
+    def __rsub__(self,value):
+        return -self + value 
 
     def __len__(self):
         return self.rows
@@ -100,29 +115,66 @@ class Vector:
             the dot product is returned
         """
         empty = []
-        if isNumber(value) or type(value) == complex or type(value) == type(epsilon): #Scalar
-            for item in self.matrix:
-                empty.append(value*item)
-            return Vector(empty)
+
+        #Scalar or anything else
+        if type(value) not in (type(self),Matrix):
+            try: 
+                for item in self.matrix:
+                    empty.append(value*item)
+                return Vector(empty)
+            except Exception:
+                raise ValueError(EXCEPTION_MSG_v("__mul__"))
+
+        #Vector of same dimensions
         elif type(value) == type(self):
             if value.getSize() != self.getSize():
                 raise ValueError("Cannot multiply non equal-size collumns ({} with {})".format(value.getSize(),self.getSize()))
             for num in range(self.getSize()):
                 empty.append(value.getMatrix()[num] * self.getMatrix()[num])
             return sum(empty)
+
+        #Another Matrix
         elif type(value) == Matrix:
             vector_to_matrix = Matrix([[item] for item in self.getMatrix()])
             return vector_to_matrix * value
 
         return NotImplemented #Redefine with __rmul__
+    
+    def __div__(self,value):
+        try:
+            return (1/value)*self
+        except:
+            raise ValueError(EXCEPTION_MSG_v("__div__"))
+
+    def __truediv__(self,value):
+        return self.__div__(value)
+
+    def __rdiv__(self,value): 
+        try:
+            return self.forEach(lambda y : value / y)
+        except:
+            raise ValueError("__rdiv__")
+
+    def __rtruediv__(self,value):
+        return self.__rdiv__(value)
+
+    def __pow__(self,value):
+        try:
+            return self.forEach(lambda y : y**value)
+        except:
+            raise ValueError(EXCEPTION_MSG_v("__pow__"))
+
+    def __rpow__(self,value):
+        try:
+            return self.forEach(lambda y : value**y)
+        except:
+            raise ValueError(EXCEPTION_MSG_v("__rpow__"))
 
     def __neg__(self):
         return (-1) * self
 
     def __rmul__(self,scalar : Union[int,float]):
-        if type(scalar) in (int,float,complex,POLY):
-            return self.__mul__(scalar)
-        raise TypeError("Cannot perform '*' operation on Vector with {}")
+        return self.__mul__(scalar)
 
     def __round__(self,ndigits : int = 1):
         __tmp__ : list = []
@@ -146,7 +198,10 @@ class Vector:
     def AngleVector(self,vector1 : "Vector",degrees : bool = False) -> float:
         return AngleBetweenVectors(self,vector1,degrees)
 
-    # def cross(self,Vector)
+    def forEach(self,function : callable) -> 'Vector':
+        return Vector([
+            function(element) for element in self.getMatrix()
+        ])
 
 class Matrix:
     """
@@ -205,9 +260,6 @@ class Matrix:
         for row in matrix:
             if type(row) == list:
                 self.ROW_LENGTHS.append(len(row))
-                for num in row:
-                    if not isNumber(num) and not type(num) == type(epsilon) and not type(num) == complex:
-                        raise TypeError("Row : {} , does not contain an argument or {} or {} but instead {}!".format(matrix.index(row),type(1),type(1.0),type(num)))
             else:
                 raise ValueError("Every argument inside the base array which is considered as a row should be of type {} (Your array had at least one element that was not of type {})".format(list,list))
         if len(self.ROW_LENGTHS) != self.ROW_LENGTHS.count(self.ROW_LENGTHS[0]):
@@ -219,7 +271,7 @@ class Matrix:
         self.isSquare = self.rows == self.collumns
 
         self.cols = []
-        for j in range(self.ROW_LENGTHS[0]):
+        for _ in range(self.ROW_LENGTHS[0]):
             self.cols.append([])
 
         for row in self.matrix:
@@ -237,6 +289,7 @@ class Matrix:
         return self.cols[index]
     
     def is_square(self) -> bool:
+        """Wheter a matrix has the same number of rows as collumns"""
         return self.isSquare
 
     def collsAll(self) -> list:
@@ -256,12 +309,14 @@ class Matrix:
         return (self.rows,self.collumns) # (number of rows,number of collumns)
 
     def __eq__(self,value):
+        """Return equality if the arrays are equal"""
         if type(value) in (type(self),Vector):
             array_item = value.rawMatrix() if type(value) == type(self) else value.getMatrix()
             return self.rawMatrix() == array_item
         return NotImplemented
 
     def __str__(self):
+        """The method called when printing a matrix"""
         print("")
         x = [item[:] for item in self.matrix]
         k = 0
@@ -312,7 +367,7 @@ class Matrix:
                 if "..." in str(val):
                     NEW_ARRAY.append(val)
                     continue
-                if not 'deg' in str(val):
+                if not 'deg' in str(val) and not 'Multivariable' in str(val):
                     val = complex(val)
 
                     if val.imag != 0:                    
@@ -343,12 +398,15 @@ class Matrix:
                     ws_pol = str(val).split(":")[-1]
                     no_ws_pol = re.sub(r"\s+",r"",ws_pol)
                     finale = f'({no_ws_pol})'
-                    if len(finale) <= 7:
-                        NEW_ARRAY.append(f'({no_ws_pol})')
-                    elif 7 < len(finale) <= 9 :
-                        NEW_ARRAY.append(f'{no_ws_pol}')
+                    if not 'Multivariable' in str(val):
+                        if len(finale) <= 7:
+                            NEW_ARRAY.append(f'({no_ws_pol})')
+                        elif 7 < len(finale) <= 9 :
+                            NEW_ARRAY.append(f'{no_ws_pol}')
+                        else:
+                            NEW_ARRAY.append(f'({no_ws_pol})')
                     else:
-                        NEW_ARRAY.append(f'({no_ws_pol})')
+                        NEW_ARRAY.append(f'{no_ws_pol}')
 
             
             cols_t = " ".join(["{: <10}" for _ in range(len(NEW_ARRAY))])
@@ -357,6 +415,7 @@ class Matrix:
         return f'\n{self.rows} x {self.collumns} Matrix\n'
 
     def __round__(self,ndigits : int = 1) -> "Matrix":
+        """Method for rounding a Matrix"""
         __tmp__ : list = [[] for item in self.rawMatrix()]
         i = 0
         for row in self.rawMatrix():
@@ -370,79 +429,184 @@ class Matrix:
             i+=1
         return Matrix(__tmp__)
 
-
     def __rmul__(self,scalar):
-        """Matrix multiplication by scalar"""
-        if type(scalar) in (int,float,complex,type(epsilon)):
-            new_matrix = [[] for i in range(self.rows)] #Add the rows
-            i = 0
-            for row in self.matrix:
-                for constant in row:
-                    new_matrix[i].append(constant * scalar)
-                i+=1
-            return Matrix(new_matrix)
-        elif type(scalar) == Vector:
-            return self.__mul__(adjugate(Matrix(scalar.getMatrix())))
-        raise TypeError("You may only multiply a {} object with either a {} or a {}".format(type(self),int,float))
+        """Matrix multiplication by scalar or Matrix (rside)"""
 
+        #Multiply Every element of the Matrix by the scalar
+        if type(scalar) != type(self):
+            #Special case where it is a vector
+            if type(scalar) == Vector:
+                return self.__mul__(adjugate(Matrix(scalar.getMatrix())))
+            try:
+                new_matrix = [[] for i in range(self.rows)] #Add the rows
+                i = 0
+                for row in self.matrix:
+                    for constant in row:
+                        new_matrix[i].append(constant * scalar)
+                    i+=1
+                return Matrix(new_matrix)
+            except:
+                raise ValueError(EXCEPTION_MSG("__mul__"))
+
+        #Type is Matrix
+        else:
+            return self.__mul__(scalar)
+                
     def __neg__(self):
+        """return -Matrix"""
         return (-1) * self
     
     def __add__(self,Matrx):
+        """
+        Return the sum beetween two Matrices,
+        Even though not Mathematically defined, adding a scalar to a Matrix will apply the .forEach method,
+        since it is very commonly used in operations
+        """
+        #Scalar Operations call .forEach
         if type(Matrx) != type(self):
-            raise ValueError("A Matrix may only be added with another Matrix not {}!".format(type(Matrx)))
+            try:
+                return self.forEach(lambda item : item + Matrx)
+            except:
+                raise ValueError(EXCEPTION_MSG("__add__"))
+
+        #Row-Collumn Equality (Matrix Addition)
         if self.__len__() != Matrx.__len__():
             raise ValueError("Rows and Collumns must be equal! {} != {}".format(self.__len__(),Matrx.__len__()))
         new_matrix = [[] for row in range(self.rows)]
-        i = 0
-        for row in self.matrix:
-            k = 0 
-            for num in row:
-                new_matrix[i].append(num+Matrx.rawMatrix()[i][k])
-                k +=1
-            i+=1
-        return Matrix(new_matrix)
+        try:
+            i = 0
+            for row in self.matrix:
+                k = 0 
+                for num in row:
+                    new_matrix[i].append(num+Matrx.rawMatrix()[i][k])
+                    k +=1
+                i+=1
+            return Matrix(new_matrix)
+        except:
+            raise ValueError(EXCEPTION_MSG("__add__"))
+
+    def __radd__(self,value):
+        return self.__add__(value)
+
+    def __rsub__(self,value):
+        return -self + value
 
     def __sub__(self,Matrx):
+        """
+        Return the difference beetween two Matrices,
+        Even though not Mathematically defined, subtracting a scalar from a Matrix will apply the .foreach method,
+        since it is very commonly used in operations
+        """
+        #Even though not Mathematically defined subtracting a scalar from a Matrix will apply the .foreach method
         if type(Matrx) != type(self):
-            raise ValueError("A Matrix may only be added with another Matrix not {}!".format(type(Matrx)))
+            scalar = Matrx #Identify the value as a scalar
+            try:
+                return self.forEach(lambda item : item - scalar)
+            except:
+                raise ValueError(EXCEPTION_MSG("__sub__"))
+
+        #Rows and Collumns must be equal in order to add Matrices    
         if self.__len__() != Matrx.__len__():
             raise ValueError("Rows and Collumns must be equal! {} != {}".format(self.__len__(),Matrx.__len__()))
-        new_matrix = [[] for row in range(self.rows)]
-        i = 0
-        for row in self.matrix:
-            k = 0 
-            for num in row:
-                new_matrix[i].append(num-Matrx.rawMatrix()[i][k])
-                k +=1
-            i+=1
-        return Matrix(new_matrix)
+        try:
+            new_matrix = [[] for row in range(self.rows)]
+            i = 0
+            for row in self.matrix:
+                k = 0 
+                for num in row:
+                    new_matrix[i].append(num-Matrx.rawMatrix()[i][k])
+                    k +=1
+                i+=1
+            return Matrix(new_matrix)
+        except:
+            raise ValueError(EXCEPTION_MSG("__sub__"))
 
     def __mul__(self,value):
-        if type(value) in (int,float,complex,type(epsilon)):
+        """
+        Matrix multiplication by another Matrix or scalar
+        """
+        #Any other value or scalar
+        if type(value) not in (Vector,type(self)):
             return self.__rmul__(value)
-        if type(value) == Vector:
+        
+        #Vector
+        elif type(value) == Vector:
             vector_to_matrix = Matrix([[item] for item in value.getMatrix()])
             return self * vector_to_matrix
-        row_0 = self.__len__()
-        col_0 = value.__len__()
-        if row_0[1] != col_0[0]: 
-            raise ValueError(f"\nCannot multiply a {row_0[0]} x {row_0[1]} with a {col_0[0]} x {col_0[1]} Matrix,\nMatrix 1 must have the same number of rows as the number of collumns in Matrix 2 \n({row_0[1]} != {col_0[0]})")
-        new_matrix = [[] for i in range(row_0[0])]
-        COLS_M2 = value.collsAll()
-        j = 0
-        for row in self.matrix:
-            for collumn in COLS_M2:
-                iterations = 0
-                total = 0
-                for scalar in collumn:
-                    total += scalar*row[iterations]
-                    iterations+=1
-                new_matrix[j].append(total)
-            j+=1
-        return Matrix(new_matrix)
+        
+        #Matrix Multiplication
+        else:
+            row_0 = self.__len__()
+            col_0 = value.__len__()
+            if row_0[1] != col_0[0]: 
+                raise ValueError(f"\nCannot multiply a {row_0[0]} x {row_0[1]} with a {col_0[0]} x {col_0[1]} Matrix,\nMatrix 1 must have the same number of rows as the number of collumns in Matrix 2 \n({row_0[1]} != {col_0[0]})")
+            try:
+                new_matrix = [[] for i in range(row_0[0])]
+                COLS_M2 = value.collsAll()
+                j = 0
+                for row in self.matrix:
+                    for collumn in COLS_M2:
+                        iterations = 0
+                        total = 0
+                        for scalar in collumn:
+                            total += scalar*row[iterations]
+                            iterations+=1
+                        new_matrix[j].append(total)
+                    j+=1
+                return Matrix(new_matrix)
+            except:
+                raise EXCEPTION_MSG("__mul__")
+
+    def __div__(self,scalar):
+        """Division by scalar (inverse of scalar time the matrix)"""
+        if type(scalar) != type(self):
+            try:
+                return (1 / scalar) * self
+            except:
+                raise ValueError(EXCEPTION_MSG("__div__"))
+        return NotImplemented
+
+    def __rdiv__(self,value):
+        try:
+            return self.forEach(lambda x : value / x)
+        except:
+            raise ValueError(EXCEPTION_MSG("__rdiv__"))
+
+    def __truediv__(self,value):
+        """Division by scalar"""
+        return self.__div__(value)
+
+    def __rtruediv__(self, value):
+        return self.__rdiv__(value)
+
+    def __pow__(self,value):
+        if type(value) != type(self):
+            try:
+                return self.forEach(lambda x : x**value)
+            except:
+                raise ValueError(EXCEPTION_MSG("__pow__"))
+        return NotImplemented
+
+    def __rpow__(self,value):
+        if type(value) == type(self):
+            return NotImplemented
+        try:
+            return self.forEach(lambda x : value**x)
+        except:
+            raise ValueError(EXCEPTION_MSG("__rpow__"))
 
     def __getitem__(self,index):
+        """Return an element of the matrix
+
+            A = Matrix([
+                [1,2,3],
+                [4,5,6],
+                [7,8,9]
+            ])
+
+            In [1]: A[0][2]
+            Out[1]: 3  
+        """
         return self.rawMatrix()[index]
 
     def appendCollumn(self,collumn : list) -> None:
@@ -629,7 +793,7 @@ def MatrixOfMinors(matrix : Matrix) -> Matrix:
         i = 0 
         reduced = [item[:] for item in matrix_array]
         reduced.pop(j)
-        for num in row:
+        for _ in row:
             x = removeCollumn(Matrix(reduced),i)
             DETERMINANTS[j].append(determinant(x))
             i+=1
@@ -974,7 +1138,6 @@ def solveREF(matrix : Matrix,unknowns : Union[tuple,list],Output: Vector, onFail
         m+=1
     return result
 
-
 def Vector0(dimensions : int) -> Vector:
     return Vector([0 for _ in range(dimensions)])
 
@@ -1031,7 +1194,6 @@ def EigenVectors(square_maxtirx : Matrix,iterations : int = 50) -> Dict[Union[co
         eigen_values_vectors[root] = eigen_vector #Eigen value has a coressponding Vector
     return eigen_values_vectors
 
-
 def magnitude(vector : Vector) -> Union[float,int]:
     components = []
     for i in range(len(vector)):
@@ -1044,5 +1206,23 @@ def AngleBetweenVectors(vector_0 : Vector,vector_1 : Vector, degrees  : bool = F
     div : float = a.dot(b) / (magnitude(a) * magnitude(b))
     return arccos(div,degrees=degrees)
 
+def randomMatrix(size : tuple) -> Matrix:
+    """Generates a random matrix
+      given a tuple in the format
+      (rows,collumns)
+    """
+    s = [[] for _ in range(size[1])]
+    for _ in range(size[0]):
+        i = 0
+        for __ in range(size[1]):
+            s[i].append(rn.random())
+            i+=1
+    return Matrix(s)
+
+def randomVector(size : int) -> Vector:
+    return Vector([
+        rn.random() for _ in range(size)
+    ])
+
 if __name__ == "__main__":
-    pass    
\ No newline at end of file
+    pass
\ No newline at end of file
